import collections

import pytest
from mock import patch

from honcho.export.runit import Export

FakeProcess = collections.namedtuple('FakeProcess', 'name env')

FIX_1PROC = [FakeProcess('web.1', {'FOO': 'bar'})]


class TestExportRunit(object):
    @pytest.fixture(autouse=True)
    def exporter(self, request):
        self.export = Export()

        get_template_patcher = patch.object(Export, 'get_template')
        self.get_template = get_template_patcher.start()
        request.addfinalizer(get_template_patcher.stop)

    def test_render(self):
        out = list(self.export.render(FIX_1PROC, {'app': 'elephant'}))

        run = _files_named('elephant-web-1/run', out)
        log = _files_named('elephant-web-1/log/run', out)
        assert len(run) == 1
        assert run[0].executable
        assert len(log) == 1
        assert log[0].executable

    def test_render_env(self):
        out = list(self.export.render(FIX_1PROC, {'app': 'elephant'}))

        env = _files_named('elephant-web-1/env/FOO', out)
        assert len(env) == 1
        assert env[0].content == 'bar'


def _files_named(name, lst):
    return [f for f in lst if f.name == name]



import re
import subprocess
import sys


def test_runpy_invoke():
    """
    Ensure honcho can also be invoked using runpy (python -m)
    """
    cmd = [sys.executable, '-m', 'honcho', 'version']
    output = subprocess.check_output(cmd, universal_newlines=True)
    assert re.match(r'honcho \d\.\d\.\d.*\n', output)



import os
import shutil
import subprocess
import tempfile
from subprocess import PIPE

import pytest


class TestEnv(object):
    def __init__(self, files=None):
        self.root = None
        self._files = files

    def cleanup(self):
        if self.root is not None:
            shutil.rmtree(self.root)
            self.root = None

    def setup(self):
        self.cleanup()

        self.root = tempfile.mkdtemp()
        if self._files is not None:
            for n, contents in self._files.items():
                with open(os.path.join(self.root, n), 'w') as fp:
                    fp.write(contents)

    def path(self, *args):
        return os.path.join(self.root, *args)

    def run_honcho(self, args):
        result = subprocess.run(
            ['honcho', *args],
            cwd=self.root,
            stdout=PIPE,
            stderr=PIPE,
            universal_newlines=True)

        return result.returncode, result.stdout, result.stderr

    def run(self, *args, **kwargs):
        options = {
            'cwd': self.root,
            'stdout': PIPE,
            'stderr': PIPE,
            'universal_newlines': True,
        }
        options.update(kwargs)

        return subprocess.run(*args, **options)


@pytest.fixture
def testenv(request):
    env = TestEnv(request.param)
    env.setup()
    request.addfinalizer(env.cleanup)
    return env



import os

import pytest


@pytest.mark.parametrize('testenv', [{
    'Procfile': "web: python web.py"
}], indirect=True)
def test_export_supervisord(testenv):
    ret, out, err = testenv.run_honcho([
        'export',
        'supervisord',
        testenv.path('giraffe'),
        '-a', 'neck',
    ])

    expected = testenv.path('giraffe', 'neck.conf')

    assert ret == 0
    assert os.path.exists(expected)


@pytest.mark.parametrize('testenv', [{
    'Procfile': "web: python web.py"
}], indirect=True)
def test_export_upstart(testenv):
    ret, out, err = testenv.run_honcho([
        'export',
        'upstart',
        testenv.path('elephant'),
        '-a', 'trunk',
    ])

    assert ret == 0
    for filename in ('trunk.conf',
                     'trunk-web.conf',
                     'trunk-web-1.conf'):
        expected = testenv.path('elephant', filename)
        assert os.path.exists(expected)


@pytest.mark.parametrize('testenv', [{
    'Procfile': "web: python web.py",
    '.env': """
NORMAL=ok
SQ_SPACES='sqspace sqspace'
DQ_SPACES="dqspace dqspace"
SQ="it's got single quotes"
DQ='it has "double" quotes'
EXCL='an exclamation mark!'
SQ_DOLLAR='costs $UNINTERPOLATED amount'
DQ_DOLLAR="costs $UNINTERPOLATED amount"
"""
}], indirect=True)
def test_export_upstart_environment(testenv):
    ret, out, err = testenv.run_honcho([
        'export',
        'upstart',
        testenv.path('test'),
        '-a', 'envvars',
    ])

    assert ret == 0

    lines = open(testenv.path('test', 'envvars-web-1.conf')).readlines()
    assert 'env NORMAL=ok\n' in lines
    assert "env SQ_SPACES='sqspace sqspace'\n" in lines
    assert "env DQ_SPACES='dqspace dqspace'\n" in lines
    assert "env SQ='it'\"'\"'s got single quotes'\n" in lines
    assert "env DQ='it has \"double\" quotes'\n" in lines
    assert "env EXCL='an exclamation mark!'\n" in lines
    assert "env SQ_DOLLAR='costs $UNINTERPOLATED amount'\n" in lines
    assert "env DQ_DOLLAR='costs $UNINTERPOLATED amount'\n" in lines



import datetime

import pytest

from honcho.process import Process


class FakeClock(object):

    def now(self):
        return datetime.datetime(2012, 8, 11, 12, 42)


class FakePopen(object):

    def __init__(self, args, **kwargs):
        self.args = args
        self.kwargs = kwargs
        self.stdout = FakeOutput()
        self.pid = 0x42
        self.returncode = 0

    def wait(self):
        return self.returncode


class FakePrinter(object):

    def __init__(self, width=0):
        self.lines = []
        self.width = width

    def write(self, message):
        self.lines.append(message)

    def got_line(self, data):
        return self.find_line(data) is not None

    def find_line(self, data):
        for line in self.lines:
            if line.data == data:
                return line
        return None


class FakeQueue(object):

    def __init__(self):
        self.messages = []

    def put(self, msg):
        self.messages.append(msg)

    def got_message(self, data):
        return self.find_message(data) is not None

    def find_message(self, data):
        for msg in self.messages:
            if msg.data == data:
                return msg
        return None


class FakeOutput(object):
    """
    FakeOutput accepts a set of lines to emit in response to a call to
    #readline(), after which it will emit b''.
    """

    def __init__(self, lines=None):
        self.lines = lines if lines is not None else []

    def readline(self):
        try:
            return self.lines.pop(0)
        except IndexError:
            return b''

    def close(self):
        pass


class TestProcess(object):

    @pytest.fixture(autouse=True)
    def queue(self):
        self.q = FakeQueue()

    def test_ctor_cmd(self):
        proc = Process('echo 123')
        assert proc.cmd == 'echo 123'

    def test_ctor_name(self):
        proc = Process('echo 123', name='foo')
        assert proc.name == 'foo'

    def test_ctor_colour(self):
        proc = Process('echo 123', colour='red')
        assert proc.colour == 'red'

    def test_ctor_quiet(self):
        proc = Process('echo 123', quiet=True)
        assert proc.quiet

    def test_output_receives_start_with_pid(self):
        proc = Process('echo 123')
        proc._child_ctor = FakePopen
        proc.run(self.q)
        msg = self.q.messages[0]
        assert msg.type == 'start'
        assert msg.data == {'pid': 66}

    def test_message_contains_name(self):
        proc = Process('echo 123', name="barry")
        proc._child_ctor = FakePopen
        proc.run(self.q)
        msg = self.q.messages[0]
        assert msg.name == "barry"

    def test_message_contains_time(self):
        proc = Process('echo 123')
        proc._clock = FakeClock()
        proc._child_ctor = FakePopen
        proc.run(self.q)
        msg = self.q.messages[0]
        assert msg.time == datetime.datetime(2012, 8, 11, 12, 42)

    def test_message_contains_colour(self):
        proc = Process('echo 123', colour="red")
        proc._child_ctor = FakePopen
        proc.run(self.q)
        msg = self.q.messages[0]
        assert msg.colour == "red"

    def test_output_receives_lines(self):
        def _ctor(*args, **kwargs):
            popen = FakePopen(*args, **kwargs)
            popen.stdout = FakeOutput(lines=[b"hello\n", b"world\n"])
            return popen

        proc = Process('echo 123')
        proc._child_ctor = _ctor
        proc.run(self.q)
        assert self.q.got_message(b"hello\n")
        assert self.q.got_message(b"world\n")

    def test_output_receives_lines_invalid_utf8(self):
        def _ctor(*args, **kwargs):
            popen = FakePopen(*args, **kwargs)
            popen.stdout = FakeOutput(lines=[b"\xfe\xff\n"])
            return popen

        proc = Process('echo 123')
        proc._child_ctor = _ctor
        proc.run(self.q)
        assert self.q.got_message(b"\xfe\xff\n")

    def test_output_does_not_receive_lines_when_quiet(self):
        def _ctor(*args, **kwargs):
            popen = FakePopen(*args, **kwargs)
            popen.stdout = FakeOutput(lines=[b"hello\n", b"world\n"])
            return popen

        proc = Process('echo 123', quiet=True)
        proc._child_ctor = _ctor
        proc.run(self.q)
        assert not self.q.got_message(b"hello\n")
        assert not self.q.got_message(b"world\n")

    def test_output_receives_stop(self):
        proc = Process('echo 123')
        proc._child_ctor = FakePopen
        proc.run(self.q)
        msg = self.q.messages[-1]
        assert msg.type == 'stop'

    def test_output_receives_stop_with_returncode(self):
        def _ctor(*args, **kwargs):
            popen = FakePopen(*args, **kwargs)
            popen.returncode = 42
            return popen

        proc = Process('echo 123')
        proc._child_ctor = _ctor
        proc.run(self.q)
        msg = self.q.find_message({'returncode': 42})
        assert msg.type == 'stop'

    def test_cwd_passed_along(self):
        proc = Process('echo 123', cwd='fake-dir')
        proc._child_ctor = FakePopen
        proc.run(self.q)
        assert proc._child.kwargs['cwd'] == 'fake-dir'



import sys
import textwrap

import pytest

python_bin = sys.executable

script = textwrap.dedent("""
    import os
    import sys
    print(os.environ.get("ANIMAL", "elephant"))
    print("error output", file=sys.stderr)
""")


@pytest.mark.parametrize('testenv', [{
    'Procfile': 'foo: {0} test.py'.format(python_bin),
    'test.py': script,
}], indirect=True)
def test_start(testenv):
    ret, out, err = testenv.run_honcho(['start'])

    assert ret == 0
    assert 'elephant' in out
    assert 'error output' in out


@pytest.mark.parametrize('testenv', [{
    '.env': 'ANIMAL=giraffe',
    'Procfile': 'foo: {0} test.py'.format(python_bin),
    'test.py': script,
}], indirect=True)
def test_start_env(testenv):
    ret, out, err = testenv.run_honcho(['start'])

    assert 'giraffe' in out


@pytest.mark.parametrize('testenv', [{
    '.env': 'PROCFILE=Procfile.dev',
    'Procfile': 'foo: {0} test.py'.format(python_bin),
    'Procfile.dev': 'bar: {0} test_dev.py'.format(python_bin),
    'test.py': script,
    'test_dev.py': textwrap.dedent("""
        print("mongoose")
        """)
}], indirect=True)
def test_start_env_procfile(testenv):
    ret, out, err = testenv.run_honcho(['start'])

    assert 'mongoose' in out


@pytest.mark.parametrize('testenv', [{
    'Procfile': 'foo: {0} test.py'.format(python_bin),
    'test.py': 'import sys; sys.exit(42)',
}], indirect=True)
def test_start_returncode(testenv):
    ret, out, err = testenv.run_honcho(['start'])

    assert ret == 42



from honcho import colour


def test_colours():
    assert colour.red == '31'
    assert colour.intense_red == '31;1'
    assert colour.cyan == '36'
    assert colour.intense_cyan == '36;1'


def test_get_colours():
    gen = colour.get_colours()

    expect = ['cyan', 'yellow', 'green', 'magenta', 'red', 'blue']
    expect = [getattr(colour, x) for x in expect]
    actual = [next(gen) for _ in range(6)]

    assert expect == actual



# -*- coding: utf-8 -*-

import pytest
from mock import Mock, patch

from honcho.export.base import BaseExport, dashrepl, percentescape


class GiraffeExport(BaseExport):
    def get_template_loader(self):
        return 'longneck'


class TestBaseExport():
    @patch('jinja2.Environment')
    def test_env_default(self, env_mock):
        GiraffeExport()
        env_mock.assert_called_with(loader='longneck')

    @patch('jinja2.Environment')
    @patch('jinja2.FileSystemLoader')
    def test_env_template_dir(self, loader_mock, env_mock):
        BaseExport(template_dir='foo/bar')
        loader_mock.assert_called_with(['foo/bar'])
        env_mock.assert_called_with(loader=loader_mock.return_value)

    def test_get_template(self):
        fake_env = Mock()
        export = BaseExport(template_env=fake_env)
        export.get_template('foo/bar.tpl')
        fake_env.get_template.assert_called_with('foo/bar.tpl')


@pytest.mark.parametrize('name_in,name_out', (
    ('foo', 'foo'),
    ('foo.1', 'foo-1'),
    ('foo.bar.baz', 'foo-bar-baz'),
    ('foo_bar_baz', 'foo_bar_baz'),
    ('foo!bar:baz', 'foo-bar-baz'),
    ('καλημέρα.κόσμε', 'καλημέρα-κόσμε'),
))
def test_dashrepl(name_in, name_out):
    assert dashrepl(name_in) == name_out


@pytest.mark.parametrize('val_in,val_out', (
    ('foo', 'foo'),
    ('foo name.%h', 'foo name.%%h'),
    ('%one two% %three', '%%one two%% %%three'),
    ('foo%%bar', 'foo%%%%bar'),
))
def test_percentescape(val_in, val_out):
    assert percentescape(val_in) == val_out



import datetime
import multiprocessing
import queue

import pytest

from honcho.manager import SYSTEM_PRINTER_NAME, Manager
from honcho.printer import Message

HISTORIES = {
    'one': {
        'processes': {'foo': {}},
        'messages': (('foo', 'start', {'pid': 123}),
                     ('foo', 'line', b'hello, world!\n'),
                     ('foo', 'stop', {'returncode': 0})),
    },
    'two': {
        'processes': {'bar': {}, 'foo': {}},
        'messages': (('foo', 'start', {'pid': 123}),
                     ('bar', 'start', {'pid': 124}),
                     ('foo', 'line', b'process one\n'),
                     ('bar', 'line', b'process two\n'),
                     ('foo', 'stop', {'returncode': 0}),
                     ('bar', 'stop', {'returncode': 0})),
    },
    'returncode': {
        'processes': {'bar': {}, 'foo': {}},
        'messages': (('foo', 'start', {'pid': 123}),
                     ('bar', 'start', {'pid': 124}),
                     ('foo', 'stop', {'returncode': 456}),
                     ('bar', 'stop', {'returncode': 321})),
    },
    'output_after_stop': {
        'processes': {'bar': {}, 'foo': {}},
        'messages': (('foo', 'start', {'pid': 123}),
                     ('bar', 'start', {'pid': 124}),
                     ('foo', 'line', b'hi from foo\n'),
                     ('bar', 'line', b'hi from bar\n'),
                     ('foo', 'stop', {'returncode': 0}),
                     ('bar', 'line', b'fishmongers\n'),
                     ('bar', 'line', b'butchers\n'),
                     ('bar', 'stop', {'returncode': -15})),
    },
}


class FakeClock(object):

    def now(self):
        return datetime.datetime(2012, 8, 11, 12, 42)


class FakeProcessManager(object):

    def terminate(self, pid):
        pass

    def kill(self, pid):
        pass


class FakeProcess(object):

    def __init__(self, cmd, name=None, colour=None, quiet=None, env=None, cwd=None):
        self.cmd = cmd
        self.name = name
        self.colour = colour
        self.quiet = quiet
        self.env = env
        self.cwd = cwd

        self._events = None
        self._options = {}

    def run(self, events=None, ignore_signals=False):
        self._report('run', events_passed=events is not None)

    def _report(self, type, **data):
        if self._events is not None:
            data.update({'type': type,
                         'name': self.name})
            self._events.put(data)


class Harness(object):

    def __init__(self, history, manager):
        self.history = history
        self.manager = manager
        self.events_local = []

        self._q = multiprocessing.Queue()
        self._rc = multiprocessing.Value('i', -999)

    def run(self, wait=True):
        self.manager._process_ctor = self._process_ctor

        for name, options in self.history['processes'].items():
            self.manager.add_process(name,
                                     options.get('command', 'test'),
                                     options.get('quiet', False))

        def _loop(rc):
            self.manager.loop()
            rc.value = self.manager.returncode

        self._mproc = multiprocessing.Process(target=_loop, args=(self._rc,))
        self._mproc.start()

        for msg in self.history['messages']:
            self.send_manager(*msg)

        self._mproc.join()

    @property
    def manager_returncode(self):
        if self._rc.value == -999:
            return None
        return self._rc.value

    def send_manager(self, process_name, type, data, **kwargs):
        self.manager.events.put(Message(type=type,
                                        data=data,
                                        time=datetime.datetime.now(),
                                        name=process_name,
                                        colour=None))

    def fetch_events(self):
        """
        Retrieve any pending events from the queue and put them on the local
        event cache
        """
        while 1:
            try:
                self.events_local.append(self._q.get(False))
            except queue.Empty:
                break

    def find_events(self, name=None, type=None):
        self.fetch_events()
        results = []
        for event in self.events_local:
            if name is not None and event['name'] != name:
                continue
            if type is not None and event['type'] != type:
                continue
            results.append(event)
        return results

    def _process_ctor(self, *args, **kwargs):
        options = self.history['processes'][kwargs['name']]
        p = FakeProcess(*args, **kwargs)
        p._events = self._q
        p._options = options
        return p


class FakePrinter(object):

    def __init__(self, width=0):
        self.width = width
        self.lines_local = []

        self._q = multiprocessing.Queue()

    def write(self, message):
        # Called in a remote thread, so just put the message on the queue.
        self._q.put(message)

    def fetch_lines(self):
        """
        Retrieve any pending lines from the queue and put them on the local
        line cache
        """
        while 1:
            try:
                self.lines_local.append(self._q.get(False))
            except queue.Empty:
                break

    def got_line(self, data):
        return self.find_line(data) is not None

    def find_line(self, data):
        self.fetch_lines()
        for line in self.lines_local:
            if line.data == data:
                return line


class TestManager(object):
    @pytest.fixture(autouse=True)
    def printer(self):
        self.p = FakePrinter()
        self.m = Manager(printer=self.p)
        self.m._clock = FakeClock()
        self.m._procmgr = FakeProcessManager()

    def run_history(self, name, wait=True):
        self.h = Harness(HISTORIES[name], self.m)
        self.h.run(wait=wait)

    def test_init_sets_default_printer_width(self):
        assert self.p.width == len(SYSTEM_PRINTER_NAME)

    def test_add_process_updates_printer_width(self):
        self.m.add_process('interesting', 'ruby server.rb')
        assert self.p.width == len('interesting')

    def test_add_process_sets_name(self):
        proc = self.m.add_process('foo', 'ruby server.rb')
        assert proc.name == 'foo'

    def test_add_process_sets_cmd(self):
        proc = self.m.add_process('foo', 'ruby server.rb')
        assert proc.cmd == 'ruby server.rb'

    def test_add_process_sets_colour(self):
        proc = self.m.add_process('foo', 'ruby server.rb')
        assert proc.colour is not None

    def test_add_process_sets_unique_colours(self):
        p1 = self.m.add_process('foo', 'ruby server.rb')
        p2 = self.m.add_process('bar', 'python server.py')
        assert p1.colour != p2.colour

    def test_add_process_sets_quiet(self):
        proc = self.m.add_process('foo', 'ruby server.rb', quiet=True)
        assert proc.quiet

    def test_add_process_name_must_be_unique(self):
        self.m.add_process('foo', 'ruby server.rb')
        with pytest.raises(AssertionError):
            self.m.add_process('foo', 'another command')

    def test_add_process_sets_cwd(self):
        proc = self.m.add_process('foo', 'ruby server.rb', cwd='foo-dir')
        assert proc.cwd == 'foo-dir'

    def test_loop_with_empty_manager_returns_immediately(self):
        self.m.loop()

    def test_loop_calls_process_run(self):
        self.run_history('one')
        evts = self.h.find_events(type='run')
        assert len(evts) == 1
        assert evts[0]['name'] == 'foo'
        assert evts[0]['events_passed']

    def test_printer_receives_messages_in_correct_order(self):
        self.run_history('one')
        self.p.fetch_lines()
        assert self.p.lines_local[0].data == 'foo started (pid=123)\n'
        assert self.p.lines_local[1].data == b'hello, world!\n'
        assert self.p.lines_local[2].data == 'foo stopped (rc=0)\n'

    def test_printer_receives_lines_multi_process(self):
        self.run_history('two')
        l1 = self.p.find_line(b'process one\n')
        l2 = self.p.find_line(b'process two\n')
        assert l1.name == 'foo'
        assert l2.name == 'bar'

    def test_returncode_set_by_first_exiting_process(self):
        self.run_history('returncode')
        assert self.h.manager_returncode == 456

    def test_printer_receives_lines_after_stop(self):
        self.run_history('output_after_stop')
        assert self.p.got_line(b'fishmongers\n')
        assert self.p.got_line(b'butchers\n')



from itertools import groupby

import jinja2

from honcho.export.base import BaseExport, File, dashrepl


class Export(BaseExport):
    """
    Exporter for Systemd

    """

    def get_template_loader(self):
        return jinja2.PackageLoader(__name__, 'templates/systemd')

    def render(self, processes, context):
        master_tpl = self.get_template('master.target')
        process_master_tpl = self.get_template('process_master.target')
        process_tpl = self.get_template('process.service')

        app_name = context['app']
        process_groups = groupby(processes, lambda proc: proc.name.split(".")[0])
        process_groups = [
            ("-".join([app_name, group_name]),
             [("-".join([app_name, dashrepl(proc.name)]), proc) for proc in procs])
            for group_name, procs in process_groups]

        master_wants = [".".join([group[0], "target"]) for group in process_groups]
        context['master_wants'] = " ".join(master_wants)
        context['process_groups'] = process_groups
        yield File("{0}.target".format(app_name), master_tpl.render(context))

        for process_master_name, proc_groups in process_groups:
            process_master_wants = [".".join([p[0], "service"]) for p in proc_groups]
            context['process_master_wants'] = " ".join(process_master_wants)
            context['process_master_name'] = process_master_name
            yield File("{0}.target".format(process_master_name), process_master_tpl.render(context))

            for process_name, process in proc_groups:
                context['process'] = process
                yield File("{0}.service".format(process_name), process_tpl.render(context))



import collections

import pytest
from mock import MagicMock, call, patch

from honcho.export.upstart import Export

FakeProcess = collections.namedtuple('FakeProcess', 'name')

FIX_1PROC = [FakeProcess('web.1')]
FIX_NPROC = [FakeProcess('web.1'),
             FakeProcess('worker.1'),
             FakeProcess('worker.2')]


class TestExportUpstart(object):
    @pytest.fixture(autouse=True)
    def exporter(self, request):
        self.export = Export()

        self.master = MagicMock()
        self.process_master = MagicMock()
        self.process = MagicMock()

        def _get_template(name):
            if name.endswith('process_master.conf'):
                return self.process_master
            elif name.endswith('process.conf'):
                return self.process
            elif name.endswith('master.conf'):
                return self.master
            else:
                raise RuntimeError("tests don't know about that template")

        get_template_patcher = patch.object(Export, 'get_template')
        self.get_template = get_template_patcher.start()
        self.get_template.side_effect = _get_template
        request.addfinalizer(get_template_patcher.stop)

    @pytest.fixture(autouse=True)
    def file(self, request):
        file_patcher = patch('honcho.export.upstart.File')
        self.File = file_patcher.start()
        request.addfinalizer(file_patcher.stop)

    def test_render_master(self):
        out = list(self.export.render(FIX_1PROC, {'app': 'elephant'}))

        master = self.File('elephant.conf', self.master.render.return_value)
        assert master in out

        self.master.render.assert_called_once_with({'app': 'elephant'})

    def test_render_process_master(self):
        out = list(self.export.render(FIX_1PROC, {'app': 'elephant'}))

        process_master = self.File('elephant-web.conf',
                                   self.process_master.render.return_value)
        assert process_master in out

        expected = {'app': 'elephant',
                    'group_name': 'elephant-web'}
        self.process_master.render.assert_called_once_with(expected)

    def test_render_process(self):
        out = list(self.export.render(FIX_1PROC, {'app': 'elephant'}))

        process = self.File('elephant-web-1.conf',
                            self.process.render.return_value)
        assert process in out

        expected = {'app': 'elephant',
                    'group_name': 'elephant-web',
                    'process': FIX_1PROC[0]}
        self.process.render.assert_called_once_with(expected)

    def test_render_multiple_process_groups(self):
        out = list(self.export.render(FIX_NPROC, {'app': 'elephant'}))

        assert self.File('elephant-web.conf',
                         self.process_master.render.return_value) in out
        assert self.File('elephant-worker.conf',
                         self.process_master.render.return_value) in out

        expected = [call({'app': 'elephant',
                          'group_name': 'elephant-web'}),
                    call({'app': 'elephant',
                          'group_name': 'elephant-worker'})]
        assert self.process_master.render.call_args_list == expected

    def test_render_multiple_processes(self):
        out = list(self.export.render(FIX_NPROC, {'app': 'elephant'}))

        assert self.File('elephant-web-1.conf',
                         self.process.render.return_value) in out
        assert self.File('elephant-worker-1.conf',
                         self.process.render.return_value) in out
        assert self.File('elephant-worker-2.conf',
                         self.process.render.return_value) in out

        expected = [call({'app': 'elephant',
                          'group_name': 'elephant-web',
                          'process': FIX_NPROC[0]}),
                    call({'app': 'elephant',
                          'group_name': 'elephant-worker',
                          'process': FIX_NPROC[1]}),
                    call({'app': 'elephant',
                          'group_name': 'elephant-worker',
                          'process': FIX_NPROC[2]})]
        assert self.process.render.call_args_list == expected



import collections

import pytest
from mock import MagicMock, call, patch

from honcho.export.systemd import Export

FakeProcess = collections.namedtuple("FakeProcess", "name")

FIX_1PROC = [FakeProcess("web.1")]
FIX_NPROC = [FakeProcess("web.1"), FakeProcess("worker.1"), FakeProcess("worker.2")]


class TestExportSystemd(object):
    @pytest.fixture(autouse=True)
    def exporter(self, request):
        self.export = Export()

        self.master = MagicMock()
        self.process_master = MagicMock()
        self.process = MagicMock()

        def _get_template(name):
            if name.endswith("process_master.target"):
                return self.process_master
            elif name.endswith("process.service"):
                return self.process
            elif name.endswith("master.target"):
                return self.master
            else:
                raise RuntimeError("tests don't know about that template")

        get_template_patcher = patch.object(Export, "get_template")
        self.get_template = get_template_patcher.start()
        self.get_template.side_effect = _get_template
        request.addfinalizer(get_template_patcher.stop)

    @pytest.fixture(autouse=True)
    def file(self, request):
        file_patcher = patch("honcho.export.systemd.File")
        self.File = file_patcher.start()
        request.addfinalizer(file_patcher.stop)

    def test_render_master(self):
        out = list(self.export.render(FIX_1PROC, {"app": "elephant"}))

        master = self.File("elephant.target", self.master.render.return_value)
        assert master in out

        expected = {
            "app": "elephant",
            "master_wants": "elephant-web.target",
            "process_groups": [("elephant-web", [("elephant-web-1", FIX_1PROC[0])])],
            "process_master_name": "elephant-web",
            "process_master_wants": "elephant-web-1.service",
            "process": FIX_1PROC[0],
        }

        self.master.render.assert_called_once_with(expected)

    def test_render_process_master(self):
        out = list(self.export.render(FIX_1PROC, {"app": "elephant"}))

        process_master = self.File(
            "elephant-web.target", self.process_master.render.return_value
        )
        assert process_master in out

        expected = {
            "app": "elephant",
            "master_wants": "elephant-web.target",
            "process_groups": [("elephant-web", [("elephant-web-1", FIX_1PROC[0])])],
            "process_master_name": "elephant-web",
            "process_master_wants": "elephant-web-1.service",
            "process": FIX_1PROC[0],
        }
        self.process_master.render.assert_called_once_with(expected)

    def test_render_process(self):
        out = list(self.export.render(FIX_1PROC, {"app": "elephant"}))

        process = self.File("elephant-web-1.service", self.process.render.return_value)
        assert process in out

        expected = {
            "app": "elephant",
            "master_wants": "elephant-web.target",
            "process_groups": [("elephant-web", [("elephant-web-1", FIX_1PROC[0])])],
            "process_master_name": "elephant-web",
            "process_master_wants": "elephant-web-1.service",
            "process": FIX_1PROC[0],
        }
        self.process.render.assert_called_once_with(expected)

    def test_render_multiple_process_groups(self):
        out = list(self.export.render(FIX_NPROC, {"app": "elephant"}))

        assert (
            self.File("elephant-web.target", self.process_master.render.return_value)
            in out
        )
        assert (
            self.File("elephant-worker.target", self.process_master.render.return_value)
            in out
        )

        expected = [
            call(
                {
                    "app": "elephant",
                    "master_wants": "elephant-web.target elephant-worker.target",
                    "process_groups": [
                        ("elephant-web", [("elephant-web-1", FIX_NPROC[0])]),
                        (
                            "elephant-worker",
                            [
                                ("elephant-worker-1", FIX_NPROC[1]),
                                ("elephant-worker-2", FIX_NPROC[2]),
                            ],
                        ),
                    ],
                    "process_master_name": "elephant-worker",
                    "process_master_wants": "elephant-worker-1.service elephant-worker-2.service",
                    "process": FIX_NPROC[2],
                }
            ),
            call(
                {
                    "app": "elephant",
                    "master_wants": "elephant-web.target elephant-worker.target",
                    "process_groups": [
                        ("elephant-web", [("elephant-web-1", FIX_NPROC[0])]),
                        (
                            "elephant-worker",
                            [
                                ("elephant-worker-1", FIX_NPROC[1]),
                                ("elephant-worker-2", FIX_NPROC[2]),
                            ],
                        ),
                    ],
                    "process_master_name": "elephant-worker",
                    "process_master_wants": "elephant-worker-1.service elephant-worker-2.service",
                    "process": FIX_NPROC[2],
                }
            ),
        ]

        assert self.process_master.render.call_args_list == expected

    def test_render_multiple_processes(self):
        out = list(self.export.render(FIX_NPROC, {"app": "elephant"}))

        assert (
            self.File("elephant-web-1.service", self.process.render.return_value) in out
        )
        assert (
            self.File("elephant-worker-1.service", self.process.render.return_value)
            in out
        )
        assert (
            self.File("elephant-worker-2.service", self.process.render.return_value)
            in out
        )

        print(self.process.render.call_args_list)
        expected = call(
            {
                "app": "elephant",
                "master_wants": "elephant-web.target elephant-worker.target",
                "process_groups": [
                    ("elephant-web", [("elephant-web-1", FIX_NPROC[0])]),
                    (
                        "elephant-worker",
                        [
                            ("elephant-worker-1", FIX_NPROC[1]),
                            ("elephant-worker-2", FIX_NPROC[2]),
                        ],
                    ),
                ],
                "process_master_name": "elephant-worker",
                "process_master_wants": "elephant-worker-1.service elephant-worker-2.service",
                "process": FIX_NPROC[2],
            }
        )
        expected = [expected] * 3
        assert self.process.render.call_args_list == expected



import datetime

import pytest

from honcho.printer import Message, Printer


def fake_message(data, **kwargs):
    defaults = {
        'type': 'line',
        'data': data,
        'time': datetime.datetime(2012, 8, 11, 12, 42),
        'name': None,
        'colour': None,
    }
    defaults.update(kwargs)
    return Message(**defaults)


class FakeOutput(object):

    def __init__(self):
        self.out = []
        self.flushcount = 0

    def flush(self):
        self.flushcount += 1

    def write(self, data):
        self.out.append(data)

    def string(self):
        return "".join(self.out)


class FakeTTY(FakeOutput):

    def isatty(self):
        return True


class TestPrinter(object):
    def test_write(self):
        out = FakeOutput()
        p = Printer(output=out)
        p.write(fake_message("monkeys\n"))
        assert out.string() == "12:42:00 | monkeys\n"

    def test_write_wrong_type(self):
        out = FakeOutput()
        p = Printer(output=out)
        with pytest.raises(RuntimeError):
            p.write(fake_message("monkeys\n", type="error"))

    def test_write_invalid_utf8(self):
        out = FakeOutput()
        p = Printer(output=out)
        p.write(fake_message(b"\xfe\xff\n"))
        assert out.string() == "12:42:00 | \ufffd\ufffd\n"

    def test_write_no_newline(self):
        out = FakeOutput()
        p = Printer(output=out)
        p.write(fake_message("monkeys"))
        assert out.string() == "12:42:00 | monkeys\n"

    def test_write_multiline(self):
        out = FakeOutput()
        p = Printer(output=out)
        p.write(fake_message("one\ntwo\nthree\n"))
        expect = "12:42:00 | one\n12:42:00 | two\n12:42:00 | three\n"
        assert out.string() == expect

    def test_write_with_name(self):
        out = FakeOutput()
        p = Printer(output=out)
        p.write(fake_message("quiescent\n", name="Robert Louis Stevenson"))
        assert out.string() == "12:42:00 Robert Louis Stevenson | quiescent\n"

    def test_write_with_set_width(self):
        out = FakeOutput()
        p = Printer(output=out, width=6)
        p.write(fake_message("giraffe\n"))
        assert out.string() == "12:42:00        | giraffe\n"

    def test_write_with_name_and_set_width(self):
        out = FakeOutput()
        p = Printer(output=out, width=6)
        p.write(fake_message("narcissist\n", name="oop"))
        assert out.string() == "12:42:00 oop    | narcissist\n"

    def test_write_with_colour_tty(self):
        out = FakeTTY()
        p = Printer(output=out)
        p.write(fake_message("conflate\n", name="foo", colour="31"))
        assert out.string() == "\033[0m\033[31m12:42:00 foo | \033[0mconflate\n"

    def test_write_with_colour_non_tty(self):
        out = FakeOutput()
        p = Printer(output=out)
        p.write(fake_message("conflate\n", name="foo", colour="31"))
        assert out.string() == "12:42:00 foo | conflate\n"

    def test_write_without_prefix_tty(self):
        out = FakeTTY()
        p = Printer(output=out, prefix=False, colour=True)
        p.write(fake_message("paranoid android\n", name="foo", colour="31"))
        assert out.string() == "paranoid android\n"

    def test_write_without_prefix_and_colour_tty(self):
        out = FakeTTY()
        p = Printer(output=out, prefix=False, colour=False)
        p.write(fake_message("paranoid android\n", name="foo", colour="31"))
        assert out.string() == "paranoid android\n"

    def test_write_without_colour_tty(self):
        out = FakeTTY()
        p = Printer(output=out, prefix=True, colour=False)
        p.write(fake_message("paranoid android\n", name="foo", colour="31"))
        assert out.string() == "12:42:00 foo | paranoid android\n"

    def test_write_without_prefix_non_tty(self):
        out = FakeOutput()
        p = Printer(output=out, prefix=False)
        p.write(fake_message("paranoid android\n", name="foo", colour="31"))
        assert out.string() == "paranoid android\n"

    def test_write_flushes_output(self):
        out = FakeOutput()
        p = Printer(output=out, prefix=False)
        p.write(fake_message("paranoid android\n"))
        assert out.flushcount == 1



import textwrap

import pytest


@pytest.mark.parametrize('testenv', [{
    'Procfile': textwrap.dedent("""
        foo: python web.py
        bar: ruby worker.rb
        baz: node socket.js
    """)
}], indirect=True)
def test_check(testenv):
    ret, out, err = testenv.run_honcho(['check'])

    assert ret == 0
    assert 'Valid procfile detected' in err
    assert 'foo, bar, baz' in err



import textwrap

import pytest


@pytest.mark.parametrize('testenv', [{
    'Procfile': textwrap.dedent("""
        foo: python web.py
        bar: ruby worker.rb
    """)
}], indirect=True)
def test_main(testenv):
    result = testenv.run(['python', '-m', 'honcho', 'check'], check=True)

    assert 'Valid procfile detected' in result.stderr
    assert 'foo, bar' in result.stderr



"""
Compatibility layer and utilities, mostly for proper Windows and Python 3
support.
"""
import errno
import os
import signal
import sys

# This works for both 32 and 64 bit Windows
ON_WINDOWS = 'win32' in str(sys.platform).lower()

if ON_WINDOWS:
    import ctypes


class ProcessManager(object):
    if ON_WINDOWS:
        def terminate(self, pid):
            # The first argument to OpenProcess represents the desired access
            # to the process. 1 represents the PROCESS_TERMINATE access right.
            handle = ctypes.windll.kernel32.OpenProcess(1, False, pid)
            ctypes.windll.kernel32.TerminateProcess(handle, -1)
            ctypes.windll.kernel32.CloseHandle(handle)
    else:
        def terminate(self, pid):
            try:
                os.killpg(pid, signal.SIGTERM)
            except OSError as e:
                if e.errno not in [errno.EPERM, errno.ESRCH]:
                    raise

    if ON_WINDOWS:
        def kill(self, pid):
            # There's no SIGKILL on Win32...
            self.terminate(pid)
    else:
        def kill(self, pid):
            try:
                os.killpg(pid, signal.SIGKILL)
            except OSError as e:
                if e.errno not in [errno.EPERM, errno.ESRCH]:
                    raise



import pytest

from honcho import command


# All of the following groups of commands should parse to the same result
@pytest.mark.parametrize('commands', [
    # Regression test for #173
    [
        ['start', '-f', 'Procfile'],
        ['-f', 'Procfile', 'start'],
    ],
    # Regression tests for #223
    [
        ['start', '--no-prefix'],
        ['--no-prefix', 'start'],
    ],
    [
        ['start', '--no-colour'],
        ['--no-colour', 'start'],
    ],
    [
        ['start', '--no-prefix', '--no-colour'],
        ['--no-colour', 'start', '--no-prefix'],
        ['--no-prefix', 'start', '--no-colour'],
        ['--no-prefix','--no-colour', 'start'],
    ]
])
def test_command_equivalence(commands):
    if len(commands) < 2:
        pytest.fail("Must supply at least two commands")

    reference_args = commands.pop(0)
    reference_result = command.parser.parse_args(reference_args)

    for args in commands:
        result = command.parser.parse_args(args)
        assert result == reference_result


def test_port_precedence(monkeypatch):
    # Nothing specified -- should return default
    args = command.parser.parse_args(['start'])
    result = command.map_from(args)['port']
    assert result == '5000'

    # OS environment override
    monkeypatch.setenv('PORT', '4200')
    result = command.map_from(args)['port']
    assert result == '4200'

    # App environment override
    # FIXME: find a better test seam for this
    def _read_env(app_root, env):
        return {'PORT': '4300'}
    monkeypatch.setattr(command, '_read_env', _read_env)
    args = command.parser.parse_args(['start'])
    result = command.map_from(args)['port']
    assert result == '4300'

    # CLI override
    args = command.parser.parse_args(['start', '-p', '4400'])
    result = command.map_from(args)['port']
    assert result == '4400'


def test_procfile_precedence(monkeypatch):
    # Nothing specified -- should return default
    args = command.parser.parse_args(['start'])
    result = command.map_from(args)['procfile']
    assert result == 'Procfile'

    # OS environment override
    monkeypatch.setenv('PROCFILE', 'Procfile.osenv')
    result = command.map_from(args)['procfile']
    assert result == 'Procfile.osenv'

    # App environment override
    # FIXME: find a better test seam for this
    def _read_env(app_root, env):
        return {'PROCFILE': 'Procfile.appenv'}
    monkeypatch.setattr(command, '_read_env', _read_env)
    args = command.parser.parse_args(['start'])
    result = command.map_from(args)['procfile']
    assert result == 'Procfile.appenv'

    # CLI override
    args = command.parser.parse_args(['start', '-f', 'Procfile.cli'])
    result = command.map_from(args)['procfile']
    assert result == 'Procfile.cli'



# coding=utf-8

import collections
import textwrap

import pytest

from honcho import environ


@pytest.mark.parametrize('content,commands', [
    [
        """
        FOO=bar
        """,
        {'FOO': 'bar'}
    ],
    [
        """
        FOO=bar
        BAZ=qux
        """,
        {'FOO': 'bar', 'BAZ': 'qux'}
    ],
    [
        # No newline at EOF
        """
        FOO=bar""",
        {'FOO': 'bar'}
    ],
    [
        # Comments
        """
        #commented: command
        """,
        {}
    ],
    [
        # Invalid characters
        """
        -foo=command
        """,
        {}
    ],
    [
        # Single quoted
        """
        MYVAR='hello"world'
        """,
        {'MYVAR': 'hello"world'}
    ],
    [
        # Double quoted
        """
        MYVAR="hello'world"
        """,
        {'MYVAR': "hello'world"}
    ],
    [
        # Quotation mark surrounded
        r"""
        MYVAR='"surrounded"'
        """,
        {'MYVAR': '"surrounded"'}
    ],
    [
        # Escaped quotation mark surrounded
        r"""
        MYVAR=\"escaped\"
        """,
        {'MYVAR': '"escaped"'}
    ],
    [
        # At-sign in value
        r"""
        MYVAR=user@domain.com
        """,
        {'MYVAR': 'user@domain.com'}
    ],
    [
        # Much punctuation in value
        r"""
        MYVAR=~pun|u@|0n$=
        """,
        {'MYVAR': '~pun|u@|0n$='}
    ],
    [
        # Unicode values
        r"""
        MYVAR=⋃ñᴉ—☪ó∂ǝ
        """,  # noqa: RUF001
        {'MYVAR': '⋃ñᴉ—☪ó∂ǝ'}  # noqa: RUF001
    ],
    [
        # Unicode keys
        r"""
        ṀẎṾẠṚ=value
        """,
        {}
    ],
    [
        # Quoted space in value
        r"""
        MYVAR='sp ace'
        """,
        {'MYVAR': 'sp ace'}
    ],
    [
        # Escaped characters in value
        r"""
        TABS='foo\tbar'
        NEWLINES='foo\nbar'
        DOLLAR='foo\$bar'
        """,
        {'TABS': 'foo\tbar',
         'NEWLINES': 'foo\nbar',
         'DOLLAR': 'foo\\$bar'}
    ],
])
def test_environ_parse(content, commands):
    content = textwrap.dedent(content)
    result = environ.parse(content)
    assert result == commands


@pytest.mark.parametrize('content,processes', [
    [
        # Simple
        """
        web: command
        """,
        {'web': 'command'}
    ],
    [
        # Simple 2
        """
        foo: python foo.py
        bar: python bar.py
        """,
        {'foo': 'python foo.py', 'bar': 'python bar.py'}
    ],
    [
        # No newline at EOF
        """
        web: command""",
        {'web': 'command'}
    ],
    [
        # Comments
        """
        #commented: command
        """,
        {}
    ],
    [
        # Invalid characters
        """
        +foo: command
        """,
        {}
    ],
    [
        # Valid -/_ characters
        """
        -foo_bar: command
        """,
        {'-foo_bar': 'command'}
    ],
    [
        # Shell metacharacters
        """
        web: sh -c "echo $FOOBAR" >/dev/null 2>&1
        """,
        {'web': 'sh -c "echo $FOOBAR" >/dev/null 2>&1'}
    ],
])
def test_parse_procfile(content, processes):
    content = textwrap.dedent(content)
    p = environ.parse_procfile(content)
    assert p.processes == processes


def test_parse_procfile_ordered():
    content = textwrap.dedent("""
    one: onecommand
    two: twocommand
    three: twocommand
    four: fourcommand
    """)

    p = environ.parse_procfile(content)
    order = [k for k in p.processes]
    assert order == ['one', 'two', 'three', 'four']


class TestProcfile(object):
    def test_has_no_processes_after_init(self):
        p = environ.Procfile()
        assert len(p.processes) == 0

    def test_add_process(self):
        p = environ.Procfile()
        p.add_process('foo', 'echo 123')
        assert 'echo 123' == p.processes['foo']

    def test_add_process_ensures_unique_name(self):
        p = environ.Procfile()
        p.add_process('foo', 'echo 123')
        with pytest.raises(AssertionError):
            p.add_process('foo', 'echo 123')


def ep(*args, **kwargs):
    return environ.expand_processes(collections.OrderedDict(args), **kwargs)


def test_expand_processes_name():
    p = ep(("foo", "some command"))
    assert len(p) == 1
    assert p[0].name == "foo.1"


def test_expand_processes_name_multiple():
    p = ep(("foo", "some command"), ("bar", "another command"))
    assert len(p) == 2
    assert p[0].name == "foo.1"
    assert p[1].name == "bar.1"


def test_expand_processes_name_concurrency():
    p = ep(("foo", "some command"), concurrency={"foo": 3})
    assert len(p) == 3
    assert p[0].name == "foo.1"
    assert p[1].name == "foo.2"
    assert p[2].name == "foo.3"


def test_expand_processes_name_concurrency_multiple():
    p = ep(("foo", "some command"), ("bar", "another command"),
           concurrency={"foo": 3, "bar": 2})
    assert len(p) == 5
    assert p[0].name == "foo.1"
    assert p[1].name == "foo.2"
    assert p[2].name == "foo.3"
    assert p[3].name == "bar.1"
    assert p[4].name == "bar.2"


def test_expand_processes_command():
    p = ep(("foo", "some command"))
    assert p[0].cmd == "some command"


def test_expand_processes_port_not_defaulted():
    p = ep(("foo", "some command"))
    assert "PORT" not in p[0].env


def test_expand_processes_port():
    p = ep(("foo", "some command"), port=8000)
    assert p[0].env["PORT"] == "8000"


def test_expand_processes_port_multiple():
    p = ep(("foo", "some command"),
           ("bar", "another command"),
           port=8000)
    assert p[0].env["PORT"] == "8000"
    assert p[1].env["PORT"] == "8100"


def test_expand_processes_port_from_env():
    p = ep(("foo", "some command"),
           ("bar", "another command"),
           env={"PORT": 8000})
    assert p[0].env["PORT"] == "8000"
    assert p[1].env["PORT"] == "8100"


def test_expand_processes_port_from_env_coerced_to_number():
    p = ep(("foo", "some command"), env={"PORT": "5000"})
    assert p[0].env["PORT"] == "5000"


def test_expand_processes_port_from_env_overrides():
    p = ep(("foo", "some command"), env={"PORT": 5000}, port=8000)
    assert p[0].env["PORT"] == "5000"


def test_expand_processes_port_concurrency():
    p = ep(("foo", "some command"),
           ("bar", "another command"),
           concurrency={"foo": 3, "bar": 2},
           port=4000)
    assert p[0].env["PORT"] == "4000"
    assert p[1].env["PORT"] == "4001"
    assert p[2].env["PORT"] == "4002"
    assert p[3].env["PORT"] == "4100"
    assert p[4].env["PORT"] == "4101"


def test_expand_processes_quiet():
    p = ep(("foo", "some command"), quiet=["foo", "bar"])
    assert p[0].quiet


def test_expand_processes_quiet_multiple():
    p = ep(("foo", "some command"),
           ("bar", "another command"),
           quiet=["foo"])
    assert p[0].quiet
    assert not p[1].quiet


def test_expand_processes_env():
    p = ep(("foo", "some command"),
           env={"ANIMAL": "giraffe", "DEBUG": "false"})
    assert p[0].env["ANIMAL"] == "giraffe"
    assert p[0].env["DEBUG"] == "false"


def test_expand_processes_env_multiple():
    p = ep(("foo", "some command"),
           ("bar", "another command"),
           env={"ANIMAL": "giraffe", "DEBUG": "false"})
    assert p[0].env["ANIMAL"] == "giraffe"
    assert p[0].env["DEBUG"] == "false"
    assert p[1].env["ANIMAL"] == "giraffe"
    assert p[1].env["DEBUG"] == "false"


def test_set_env_process_name():
    p = ep(("foo", "some command"),
           ("bar", "another command"),
           concurrency={"foo": 3, "bar": 2})
    assert p[0].env["HONCHO_PROCESS_NAME"] == "foo.1"
    assert p[1].env["HONCHO_PROCESS_NAME"] == "foo.2"
    assert p[2].env["HONCHO_PROCESS_NAME"] == "foo.3"
    assert p[3].env["HONCHO_PROCESS_NAME"] == "bar.1"
    assert p[4].env["HONCHO_PROCESS_NAME"] == "bar.2"



import sys
import textwrap

import pytest

python_bin = sys.executable

script = textwrap.dedent("""
    import os
    import sys
    print(os.environ.get("ANIMAL", "elephant"))
    print("error output", file=sys.stderr)
""")


@pytest.mark.parametrize('testenv', [{
    'test.py': script
}], indirect=True)
def test_run(testenv):
    ret, out, err = testenv.run_honcho(['run', python_bin, 'test.py'])

    assert ret == 0
    assert out == 'elephant\n'
    assert 'error output\n' in err


@pytest.mark.parametrize('testenv', [{
    '.env': 'ANIMAL=giraffe',
    'test.py': script,
}], indirect=True)
def test_run_env(testenv):
    ret, out, err = testenv.run_honcho(['run', python_bin, 'test.py'])

    assert ret == 0
    assert out == 'giraffe\n'


@pytest.mark.parametrize('testenv', [{
    '.env.x': 'ANIMAL=giraffe',
    'test.py': script,
}], indirect=True)
def test_run_args_before_command(testenv):
    # Regression test for #122 -- ensure that common args can be given
    # before the subcommand.
    ret, out, err = testenv.run_honcho(['-e', '.env.x',
                                        'run', python_bin, 'test.py'])

    assert ret == 0
    assert out == 'giraffe\n'


@pytest.mark.parametrize('testenv', [{
    'test.py': script
}], indirect=True)
def test_run_options_positionals_separator(testenv):
    # Regression test for #159 -- ensure that honcho handles the '--'
    # options/positionals separator correctly.
    ret, out, err = testenv.run_honcho(['run', '--', python_bin, 'test.py'])

    assert ret == 0
    assert out == 'elephant\n'



from mock import MagicMock, patch

from honcho.export.supervisord import Export


class TestExportSupervisord():
    @patch.object(Export, 'get_template')
    def test_render_adds_processes_to_context(self, fake_get_template):
        export = Export()
        p1, p2 = object(), object()

        export.render([p1, p2], {'app': 'elephant'})

        fake_get_template.return_value.render.assert_called_with(
            {'app': 'elephant',
             'processes': [p1, p2]})

    def test_render_uses_app_name_as_filename(self):
        export = Export()
        export.get_template = MagicMock()

        results = export.render([], {'app': 'elephant'})

        assert results[0].name == 'elephant.conf'
